@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/include/ldif.h,v 1.23 2004/01/01 18:15:24 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/include/ldif.h,v 1.23.2.2 2005/01/24 21:28:50 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,6 +59,13 @@ ldif_parse_line LDAP_P((
 	char **value,
 	ber_len_t *vlen ));
 
+LDAP_LDIF_F( int )
+ldif_parse_line2 LDAP_P((
+	char *line,
+	struct berval *type,
+	struct berval *value,
+	int *freeval ));
+
 LDAP_LDIF_F( int )
 ldif_fetch_url LDAP_P((
 	LDAP_CONST char *line,
@@ -68,6 +75,9 @@ ldif_fetch_url LDAP_P((
 LDAP_LDIF_F( char * )
 ldif_getline LDAP_P(( char **next ));
 
+LDAP_LDIF_F( int )
+ldif_countlines LDAP_P(( LDAP_CONST char *line ));
+
 LDAP_LDIF_F( int )
 ldif_read_record LDAP_P((
 	FILE *fp,
