@@ -1,8 +1,8 @@
 /* attr.c - backend routines for dealing with attributes */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/attr.c,v 1.19.2.1 2004/12/30 21:29:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/attr.c,v 1.19.2.3 2005/01/24 21:28:52 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2004 The OpenLDAP Foundation.
+ * Copyright 2000-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -67,6 +67,24 @@ bdb_attr_comp_ref(
 	
 	*cr = a != NULL ? a->ai_cr : 0 ;
 }
+void
+bdb_attr_mask_cr(
+	struct bdb_info *bdb,
+	AttributeDescription *desc,
+	slap_mask_t *indexmask,
+	ComponentReference** cr )
+{
+	AttrInfo	*a;
+
+	a = (AttrInfo *) avl_find( bdb->bi_attrs, desc, ainfo_type_cmp );
+	if ( a ) {
+		*indexmask = a->ai_indexmask;
+		*cr = a->ai_cr;
+	} else {
+		*indexmask = NULL;
+		*cr = NULL;
+	}
+}
 #endif
 
 void
