@@ -1,8 +1,8 @@
 /* config.c - bdb backend configuration file routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/config.c,v 1.43 2004/11/25 20:16:56 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/config.c,v 1.43.2.2 2005/01/28 17:29:49 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2004 The OpenLDAP Foundation.
+ * Copyright 2000-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -63,15 +63,6 @@ bdb_db_config(
 	} else if ( strcasecmp( argv[0], "dbnosync" ) == 0 ) {
 		bdb->bi_dbenv_xflags |= DB_TXN_NOSYNC;
 
-	/* slapadd/slapindex logging configuration */
-	} else if ( strcasecmp( argv[0], "fasttool" ) == 0 ) {
-		if ( slapMode & SLAP_TOOL_MODE )
-#if DB_VERSION_FULL >= 0x04030015
-		bdb->bi_dbenv_xflags |= DB_LOG_INMEMORY;
-#else
-		bdb->bi_dbenv_xflags |= DB_TXN_NOT_DURABLE;
-#endif
-
 	/* slapindex algorithm tuning */
 	} else if ( strcasecmp( argv[0], "linearindex" ) == 0 ) {
 		bdb->bi_linear_index = 1;
@@ -194,48 +185,7 @@ bdb_db_config(
 		}
 		if ( !( slapMode & SLAP_TOOL_MODE ) )
 			bdb->bi_idl_cache_max_size = atoi( argv[1] );
-#ifdef BDB_PSEARCH
-	} else if ( strcasecmp( argv[0], "sessionlog" ) == 0 ) {
-		int se_id = 0, se_size = 0;
-		struct slap_session_entry *sent;
-		if ( argc < 3 ) {
-			Debug( LDAP_DEBUG_ANY,
-				"%s: line %d: missing arguments in \"sessionlog <id> <size>\""
-				" line\n", fname, lineno, 0 );
-			return( 1 );
-		}
 
-		se_id = atoi( argv[1] );
-
-		if ( se_id < 0 || se_id > 999 ) {
-			Debug( LDAP_DEBUG_ANY,
-				"%s: line %d: session log id %d is out of range [0..999]\n",
-				fname, lineno , se_id );
-			return( 1 );
-		}
-
-		se_size = atoi( argv[2] );
-		if ( se_size < 0 ) {
-			Debug( LDAP_DEBUG_ANY,
-				"%s: line %d: session log size %d is negative\n",
-				fname, lineno , se_size );
-			return( 1 );
-		}
-
-		LDAP_LIST_FOREACH( sent, &bdb->bi_session_list, se_link ) {
-			if ( sent->se_id == se_id ) {
-				Debug( LDAP_DEBUG_ANY,
-					"%s: line %d: session %d already exists\n",
-					fname, lineno , se_id );
-				return( 1 );
-			}
-		}
-		sent = (struct slap_session_entry *) ch_calloc( 1,
-						sizeof( struct slap_session_entry ));
-		sent->se_id = se_id;
-		sent->se_size = se_size;
-		LDAP_LIST_INSERT_HEAD( &bdb->bi_session_list, sent, se_link );
-#endif /* BDB_PSEARCH */
 	/* anything else */
 	} else {
 		return SLAP_CONF_UNKNOWN;
