@@ -1,8 +1,8 @@
 /* index.c - routines for dealing with attribute indexes */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/index.c,v 1.46.2.1 2004/12/30 21:29:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/index.c,v 1.46.2.3 2005/01/24 21:28:52 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2004 The OpenLDAP Foundation.
+ * Copyright 2000-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -241,7 +241,7 @@ static int indexer(
 
 		if( rc == LDAP_SUCCESS && keys != NULL ) {
 			for( i=0; keys[i].bv_val != NULL; i++ ) {
-				bdb_key_change( op->o_bd, db, txn, &keys[i], id, opid );
+				rc = bdb_key_change( op->o_bd, db, txn, &keys[i], id, opid );
 				if( rc ) {
 					ber_bvarray_free_x( keys, op->o_tmpmemctx );
 					goto done;
@@ -283,20 +283,21 @@ static int index_at_values(
 		if( rc ) return rc;
 	}
 
-#ifdef LDAP_COMP_MATCH
-	/* component indexing */
-	bdb_attr_comp_ref ( op->o_bd->be_private, type->sat_ad, &cr_list );
-	if ( cr_list ) {
-		for( cr = cr_list ; cr ; cr = cr->cr_next ) {
-			rc = indexer( op, txn, cr->cr_ad, &type->sat_cname,
-				cr->cr_nvals, id, opid,
-				cr->cr_indexmask );
-		}
-	}
-#endif
 	/* If this type has no AD, we've never used it before */
 	if( type->sat_ad ) {
+#ifdef LDAP_COMP_MATCH
+		/* component indexing */
+		bdb_attr_mask_cr( op->o_bd->be_private, type->sat_ad, &mask, &cr_list );
+		if ( cr_list ) {
+			for( cr = cr_list ; cr ; cr = cr->cr_next ) {
+				rc = indexer( op, txn, cr->cr_ad, &type->sat_cname,
+					cr->cr_nvals, id, opid,
+					cr->cr_indexmask );
+			}
+		}
+#else
 		bdb_attr_mask( op->o_bd->be_private, type->sat_ad, &mask );
+#endif
 		ad = type->sat_ad;
 	}
 
