@@ -1,8 +1,8 @@
 /* bind.c - ldap backend bind function */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/bind.c,v 1.85 2004/12/08 18:11:27 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/bind.c,v 1.85.2.4 2005/02/02 17:32:44 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Copyright 1999-2005 The OpenLDAP Foundation.
  * Portions Copyright 2000-2003 Pierangelo Masarati.
  * Portions Copyright 1999-2003 Howard Chu.
  * All rights reserved.
@@ -16,7 +16,7 @@
  * <http://www.OpenLDAP.org/license.html>.
  */
 /* ACKNOWLEDGEMENTS:
- * This work was initially developed by the Howard Chu for inclusion
+ * This work was initially developed by Howard Chu for inclusion
  * in OpenLDAP Software and subsequently enhanced by Pierangelo
  * Masarati.
  */
@@ -38,10 +38,11 @@
 
 static LDAP_REBIND_PROC	ldap_back_rebind;
 
-#ifdef LDAP_BACK_PROXY_AUTHZ
 static int
 ldap_back_proxy_authz_bind( struct ldapconn *lc, Operation *op, SlapReply *rs );
-#endif /* LDAP_BACK_PROXY_AUTHZ */
+
+static int
+ldap_back_prepare_conn( struct ldapconn **lcp, Operation *op, SlapReply *rs, ldap_back_send_t sendok );
 
 int
 ldap_back_bind( Operation *op, SlapReply *rs )
@@ -52,9 +53,9 @@ ldap_back_bind( Operation *op, SlapReply *rs )
 	int rc = 0;
 	ber_int_t msgid;
 
-	lc = ldap_back_getconn( op, rs );
+	lc = ldap_back_getconn( op, rs, LDAP_BACK_SENDERR );
 	if ( !lc ) {
-		return( -1 );
+		return rs->sr_err;
 	}
 
 	if ( !BER_BVISNULL( &lc->lc_bound_ndn ) ) {
@@ -67,10 +68,14 @@ ldap_back_bind( Operation *op, SlapReply *rs )
 	rs->sr_err = ldap_sasl_bind( lc->lc_ld, op->o_req_dn.bv_val,
 			LDAP_SASL_SIMPLE,
 			&op->orb_cred, op->o_ctrls, NULL, &msgid );
-	rc = ldap_back_op_result( lc, op, rs, msgid, 1 );
+	rc = ldap_back_op_result( lc, op, rs, msgid, LDAP_BACK_SENDERR );
 
 	if ( rc == LDAP_SUCCESS ) {
-#if defined(LDAP_BACK_PROXY_AUTHZ)
+		/* If defined, proxyAuthz will be used also when
+		 * back-ldap is the authorizing backend; for this
+		 * purpose, a successful bind is followed by a
+		 * bind with the configured identity assertion */
+		/* NOTE: use with care */
 		if ( li->idassert_flags & LDAP_BACK_AUTH_OVERRIDE ) {
 			ldap_back_proxy_authz_bind( lc, op, rs );
 			if ( lc->lc_bound == 0 ) {
@@ -78,12 +83,11 @@ ldap_back_bind( Operation *op, SlapReply *rs )
 				goto done;
 			}
 		}
-#endif /* LDAP_BACK_PROXY_AUTHZ */
 
 		lc->lc_bound = 1;
 		ber_dupbv( &lc->lc_bound_ndn, &op->o_req_ndn );
 
-		if ( li->savecred ) {
+		if ( li->flags & LDAP_BACK_F_SAVECRED ) {
 			if ( !BER_BVISNULL( &lc->lc_cred ) ) {
 				memset( lc->lc_cred.bv_val, 0,
 						lc->lc_cred.bv_len );
@@ -218,12 +222,73 @@ ldap_back_freeconn( Operation *op, struct ldapconn *lc )
 	return 0;
 }
 
+static int
+ldap_back_prepare_conn( struct ldapconn **lcp, Operation *op, SlapReply *rs, ldap_back_send_t sendok )
+{
+	struct ldapinfo	*li = (struct ldapinfo *)op->o_bd->be_private;
+	int		vers = op->o_protocol;
+	LDAP		*ld = NULL;
+
+	assert( lcp != NULL );
+
+	rs->sr_err = ldap_initialize( &ld, li->url );
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		goto error_return;
+	}
+
+	/* Set LDAP version. This will always succeed: If the client
+	 * bound with a particular version, then so can we.
+	 */
+	ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION, (const void *)&vers );
+
+	/* automatically chase referrals ("chase-referrals"/"dont-chase-referrals" statement) */
+	if ( li->flags & LDAP_BACK_F_CHASE_REFERRALS ) {
+		ldap_set_option( ld, LDAP_OPT_REFERRALS, LDAP_OPT_ON );
+	}
+
+	/* start TLS ("start-tls"/"try-start-tls" statements) */
+	if ( ( li->flags & LDAP_BACK_F_USE_TLS )
+			&& !ldap_is_ldaps_url( li->url )
+			&& ( rs->sr_err = ldap_start_tls_s( ld, NULL, NULL ) ) != LDAP_SUCCESS )
+	{
+		/* if StartTLS is requested, only attempt it if the URL
+		 * is not "ldaps://"; this may occur not only in case
+		 * of misconfiguration, but also when used in the chain 
+		 * overlay, where the "uri" can be parsed out of a referral */
+		if ( rs->sr_err == LDAP_SERVER_DOWN
+				|| ( li->flags & LDAP_BACK_F_TLS_CRITICAL ) )
+		{
+			ldap_unbind_ext_s( ld, NULL, NULL );
+			goto error_return;
+		}
+	}
+
+	if ( *lcp == NULL ) {
+		*lcp = (struct ldapconn *)ch_malloc( sizeof( struct ldapconn ) );
+		memset( *lcp, 0, sizeof( struct ldapconn ) );
+	}
+	(*lcp)->lc_ld = ld;
+
+error_return:;
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		rs->sr_err = slap_map_api2result( rs );
+		if ( sendok & LDAP_BACK_SENDERR ) {
+			if ( rs->sr_text == NULL ) {
+				rs->sr_text = "ldap_initialize() failed";
+			}
+			send_ldap_result( op, rs );
+			rs->sr_text = NULL;
+		}
+	}
+
+	return rs->sr_err;
+}
+
 struct ldapconn *
-ldap_back_getconn( Operation *op, SlapReply *rs )
+ldap_back_getconn( Operation *op, SlapReply *rs, ldap_back_send_t sendok )
 {
 	struct ldapinfo	*li = (struct ldapinfo *)op->o_bd->be_private;
 	struct ldapconn	*lc, lc_curr;
-	LDAP		*ld;
 	int		is_priv = 0;
 
 	/* Searches for a ldapconn in the avl tree */
@@ -257,31 +322,12 @@ ldap_back_getconn( Operation *op, SlapReply *rs )
 
 	/* Looks like we didn't get a bind. Open a new session... */
 	if ( !lc ) {
-		int vers = op->o_protocol;
-		rs->sr_err = ldap_initialize( &ld, li->url );
-		
-		if ( rs->sr_err != LDAP_SUCCESS ) {
-			rs->sr_err = slap_map_api2result( rs );
-			if ( rs->sr_text == NULL ) {
-				rs->sr_text = "ldap_initialize() failed";
-			}
-			if ( op->o_conn ) {
-				send_ldap_result( op, rs );
-			}
-			rs->sr_text = NULL;
-			return( NULL );
+		/* lc here must be NULL */
+		if ( ldap_back_prepare_conn( &lc, op, rs, sendok ) != LDAP_SUCCESS ) {
+			return NULL;
 		}
-		/* Set LDAP version. This will always succeed: If the client
-		 * bound with a particular version, then so can we.
-		 */
-		ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION,
-				(const void *)&vers );
-		/* FIXME: configurable? */
-		ldap_set_option( ld, LDAP_OPT_REFERRALS, LDAP_OPT_ON );
 
-		lc = (struct ldapconn *)ch_malloc( sizeof( struct ldapconn ) );
 		lc->lc_conn = lc_curr.lc_conn;
-		lc->lc_ld = ld;
 		ber_dupbv( &lc->lc_local_ndn, &lc_curr.lc_local_ndn );
 
 		ldap_pvt_thread_mutex_init( &lc->lc_mutex );
@@ -319,18 +365,18 @@ ldap_back_getconn( Operation *op, SlapReply *rs )
 		/* Err could be -1 in case a duplicate ldapconn is inserted */
 		if ( rs->sr_err != 0 ) {
 			ldap_back_conn_free( lc );
-			if ( op->o_conn ) {
+			if ( op->o_conn && ( sendok & LDAP_BACK_SENDERR ) ) {
 				send_ldap_error( op, rs, LDAP_OTHER,
 				"internal server error" );
 			}
-			return( NULL );
+			return NULL;
 		}
 	} else {
 		Debug( LDAP_DEBUG_TRACE,
 			"=>ldap_back_getconn: conn %p fetched\n", (void *) lc, 0, 0 );
 	}
 	
-	return( lc );
+	return lc;
 }
 
 /*
@@ -340,15 +386,23 @@ ldap_back_getconn( Operation *op, SlapReply *rs )
  * it from all the callers, and I made the function return the flag, so
  * it can be used to simplify the check.
  */
-int
-ldap_back_dobind( struct ldapconn *lc, Operation *op, SlapReply *rs )
+static int
+ldap_back_dobind_int(
+	struct ldapconn		*lc,
+	Operation		*op,
+	SlapReply		*rs,
+	ldap_back_send_t	sendok,
+	int			retries )
 {	
 	int		rc;
 	ber_int_t	msgid;
 
+	assert( retries >= 0 );
+
 	ldap_pvt_thread_mutex_lock( &lc->lc_mutex );
 	if ( !lc->lc_bound ) {
-#ifdef LDAP_BACK_PROXY_AUTHZ
+		struct ldapinfo	*li = (struct ldapinfo *)op->o_bd->be_private;
+
 		/*
 		 * FIXME: we need to let clients use proxyAuthz
 		 * otherwise we cannot do symmetric pools of servers;
@@ -363,23 +417,49 @@ ldap_back_dobind( struct ldapconn *lc, Operation *op, SlapReply *rs )
 		 */
 		/*
 		 * if no bind took place yet, but the connection is bound
-		 * and the "proxyauthzdn" is set, then bind as 
-		 * "proxyauthzdn" and explicitly add the proxyAuthz 
-		 * control to every operation with the dn bound 
-		 * to the connection as control value.
+		 * and the "idassert-authcDN" (or other ID) is set, 
+		 * then bind as the asserting identity and explicitly 
+		 * add the proxyAuthz control to every operation with the
+		 * dn bound to the connection as control value.
+		 * This is done also if this is the authrizing backend,
+		 * but the "override" flag is given to idassert.
+		 * It allows to use SASL bind and yet proxyAuthz users
 		 */
-		if ( op->o_conn != NULL && BER_BVISNULL( &lc->lc_bound_ndn ) ) {
+		if ( op->o_conn != NULL &&
+				( BER_BVISNULL( &lc->lc_bound_ndn ) ||
+				  ( li->idassert_flags & LDAP_BACK_AUTH_OVERRIDE ) ) )
+		{
 			(void)ldap_back_proxy_authz_bind( lc, op, rs );
 			goto done;
 		}
-#endif /* LDAP_BACK_PROXY_AUTHZ */
 
+retry:;
 		rs->sr_err = ldap_sasl_bind( lc->lc_ld,
 				lc->lc_bound_ndn.bv_val,
 				LDAP_SASL_SIMPLE, &lc->lc_cred,
 				NULL, NULL, &msgid );
-		
-		rc = ldap_back_op_result( lc, op, rs, msgid, 0 );
+
+		if ( rs->sr_err == LDAP_SERVER_DOWN ) {
+			if ( retries > 0 ) {
+				ldap_unbind_ext_s( lc->lc_ld, NULL, NULL );
+				lc->lc_ld = NULL;
+
+				/* lc here must be the regular lc, reset and ready for init */
+				if ( ldap_back_prepare_conn( &lc, op, rs, sendok ) != LDAP_SUCCESS ) {
+					return 0;
+				}
+
+				retries--;
+				goto retry;
+			}
+
+			ldap_back_freeconn( op, lc );
+			rs->sr_err = slap_map_api2result( rs );
+
+			return 0;
+		}
+
+		rc = ldap_back_op_result( lc, op, rs, msgid, sendok );
 		if ( rc == LDAP_SUCCESS ) {
 			lc->lc_bound = 1;
 		}
@@ -391,6 +471,12 @@ done:;
 	return rc;
 }
 
+int
+ldap_back_dobind( struct ldapconn *lc, Operation *op, SlapReply *rs, ldap_back_send_t sendok )
+{
+	return ldap_back_dobind_int( lc, op, rs, sendok, 1 );
+}
+
 /*
  * ldap_back_rebind
  *
@@ -401,7 +487,9 @@ static int
 ldap_back_rebind( LDAP *ld, LDAP_CONST char *url, ber_tag_t request,
 	ber_int_t msgid, void *params )
 {
-	struct ldapconn *lc = params;
+	struct ldapconn *lc = (struct ldapconn *)params;
+
+	/* FIXME: add checks on the URL/identity? */
 
 	return ldap_sasl_bind_s( ld, lc->lc_bound_ndn.bv_val,
 			LDAP_SASL_SIMPLE, &lc->lc_cred, NULL, NULL, NULL );
@@ -409,11 +497,11 @@ ldap_back_rebind( LDAP *ld, LDAP_CONST char *url, ber_tag_t request,
 
 int
 ldap_back_op_result(
-		struct ldapconn	*lc,
-		Operation	*op,
-		SlapReply	*rs,
-		ber_int_t	msgid,
-		int		sendok )
+		struct ldapconn		*lc,
+		Operation		*op,
+		SlapReply		*rs,
+		ber_int_t		msgid,
+		ldap_back_send_t	sendok )
 {
 	char		*match = NULL;
 	LDAPMessage	*res = NULL;
@@ -428,17 +516,30 @@ ldap_back_op_result(
 	 * to a successful state, get the error from the
 	 * remote server response */
 	if ( ERR_OK( rs->sr_err ) ) {
+		int		rc;
+		struct timeval	tv = { 0, 0 };
+
+retry:;
 		/* if result parsing fails, note the failure reason */
-		if ( ldap_result( lc->lc_ld, msgid, 1, NULL, &res ) == -1 ) {
+		switch ( ldap_result( lc->lc_ld, msgid, 1, &tv, &res ) ) {
+		case 0:
+			tv.tv_sec = 0;
+			tv.tv_usec = 100000;	/* 0.1 s */
+			ldap_pvt_thread_yield();
+			goto retry;
+
+		case -1:
 			ldap_get_option( lc->lc_ld, LDAP_OPT_ERROR_NUMBER,
 					&rs->sr_err );
+			break;
+
 
 		/* otherwise get the result; if it is not
 		 * LDAP_SUCCESS, record it in the reply
 		 * structure (this includes 
 		 * LDAP_COMPARE_{TRUE|FALSE}) */
-		} else {
-			int rc = ldap_parse_result( lc->lc_ld, res, &rs->sr_err,
+		default:
+			rc = ldap_parse_result( lc->lc_ld, res, &rs->sr_err,
 					&match, &text, NULL, NULL, 1 );
 			rs->sr_text = text;
 			if ( rc != LDAP_SUCCESS ) {
@@ -462,7 +563,10 @@ ldap_back_op_result(
 			rs->sr_matched = match;
 		}
 	}
-	if ( op->o_conn && ( sendok || rs->sr_err != LDAP_SUCCESS ) ) {
+	if ( op->o_conn &&
+			( ( sendok & LDAP_BACK_SENDOK ) 
+			  || ( ( sendok & LDAP_BACK_SENDERR ) && rs->sr_err != LDAP_SUCCESS ) ) )
+	{
 		send_ldap_result( op, rs );
 	}
 	if ( match ) {
@@ -481,40 +585,22 @@ ldap_back_op_result(
 
 /* return true if bound, false if failed */
 int
-ldap_back_retry( struct ldapconn *lc, Operation *op, SlapReply *rs )
+ldap_back_retry( struct ldapconn *lc, Operation *op, SlapReply *rs, ldap_back_send_t sendok )
 {
-	struct ldapinfo	*li = (struct ldapinfo *)op->o_bd->be_private;
-	int vers = op->o_protocol;
-	LDAP *ld;
-
 	ldap_pvt_thread_mutex_lock( &lc->lc_mutex );
 	ldap_unbind_ext_s( lc->lc_ld, NULL, NULL );
+	lc->lc_ld = NULL;
 	lc->lc_bound = 0;
-	rs->sr_err = ldap_initialize( &ld, li->url );
-		
-	if ( rs->sr_err != LDAP_SUCCESS ) {
-		rs->sr_err = slap_map_api2result( rs );
-		if ( rs->sr_text == NULL ) {
-			rs->sr_text = "ldap_initialize() failed";
-		}
-		if ( op->o_conn ) {
-			send_ldap_result( op, rs );
-		}
-		rs->sr_text = NULL;
+
+	/* lc here must be the regular lc, reset and ready for init */
+	if ( ldap_back_prepare_conn( &lc, op, rs, sendok ) != LDAP_SUCCESS ) {
 		return 0;
 	}
-	/* Set LDAP version. This will always succeed: If the client
-	 * bound with a particular version, then so can we.
-	 */
-	ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION, (const void *)&vers );
-	/* FIXME: configurable? */
-	ldap_set_option( ld, LDAP_OPT_REFERRALS, LDAP_OPT_ON );
-	lc->lc_ld = ld;
+
 	ldap_pvt_thread_mutex_unlock( &lc->lc_mutex );
-	return ldap_back_dobind( lc, op, rs );
+	return ldap_back_dobind_int( lc, op, rs, sendok, 0 );
 }
 
-#ifdef LDAP_BACK_PROXY_AUTHZ
 static int
 ldap_back_proxy_authz_bind( struct ldapconn *lc, Operation *op, SlapReply *rs )
 {
@@ -562,8 +648,13 @@ ldap_back_proxy_authz_bind( struct ldapconn *lc, Operation *op, SlapReply *rs )
 
 	default:
 		if ( li->idassert_authz ) {
-			struct berval	authcDN = BER_BVISNULL( &op->o_conn->c_ndn ) ? slap_empty_bv : op->o_conn->c_ndn;
+			struct berval authcDN;
 
+			if ( BER_BVISNULL( &op->o_conn->c_ndn ) ) {
+				authcDN = slap_empty_bv;
+			} else {
+				authcDN = op->o_conn->c_ndn;
+			}	
 			rs->sr_err = slap_sasl_matches( op, li->idassert_authz,
 					&authcDN, &authcDN );
 			if ( rs->sr_err != LDAP_SUCCESS ) {
@@ -679,7 +770,7 @@ ldap_back_proxy_authz_bind( struct ldapconn *lc, Operation *op, SlapReply *rs )
 		goto done;
 	}
 
-	rc = ldap_back_op_result( lc, op, rs, msgid, 0 );
+	rc = ldap_back_op_result( lc, op, rs, msgid, LDAP_BACK_SENDERR );
 	if ( rc == LDAP_SUCCESS ) {
 		lc->lc_bound = 1;
 	}
@@ -724,6 +815,8 @@ ldap_back_proxy_authz_ctrl(
 
 	*pctrls = NULL;
 
+	rs->sr_err = LDAP_SUCCESS;
+
 	if ( ( BER_BVISNULL( &li->idassert_authcID ) || BER_BVISEMPTY( &li->idassert_authcID ) )
 			&& ( BER_BVISNULL( &li->idassert_authcDN ) || BER_BVISEMPTY( &li->idassert_authcDN ) ) ) {
 		goto done;
@@ -777,9 +870,13 @@ ldap_back_proxy_authz_ctrl(
 
 	} else if ( li->idassert_authz ) {
 		int		rc;
-		struct berval	authcDN = BER_BVISNULL( &op->o_conn->c_ndn ) ? slap_empty_bv : op->o_conn->c_ndn;
-
+		struct berval authcDN;
 
+		if ( BER_BVISNULL( &op->o_conn->c_ndn ) ) {
+			authcDN = slap_empty_bv;
+		} else {
+			authcDN = op->o_conn->c_ndn;
+		}
 		rc = slap_sasl_matches( op, li->idassert_authz,
 				&authcDN, & authcDN );
 		if ( rc != LDAP_SUCCESS ) {
@@ -823,7 +920,11 @@ ldap_back_proxy_authz_ctrl(
 	case LDAP_BACK_IDASSERT_SELF:
 		/* original behavior:
 		 * assert the client's identity */
-		assertedID = BER_BVISNULL( &op->o_conn->c_ndn ) ? slap_empty_bv : op->o_conn->c_ndn;
+		if ( BER_BVISNULL( &op->o_conn->c_ndn ) ) {
+			assertedID = slap_empty_bv;
+		} else {
+			assertedID = op->o_conn->c_ndn;
+		}
 		break;
 
 	case LDAP_BACK_IDASSERT_ANONYMOUS:
@@ -899,6 +1000,9 @@ ldap_back_proxy_authz_ctrl_free( Operation *op, LDAPControl ***pctrls )
 {
 	LDAPControl	**ctrls = *pctrls;
 
+	/* we assume that the first control is the proxyAuthz
+	 * added by back-ldap, so it's the only one we explicitly 
+	 * free */
 	if ( ctrls && ctrls != op->o_ctrls ) {
 		assert( ctrls[ 0 ] );
 
@@ -914,4 +1018,3 @@ ldap_back_proxy_authz_ctrl_free( Operation *op, LDAPControl ***pctrls )
 
 	return 0;
 }
-#endif /* LDAP_BACK_PROXY_AUTHZ */
