@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/back-meta.h,v 1.23.2.1 2004/12/30 21:29:58 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/back-meta.h,v 1.23.2.3 2005/01/24 21:28:52 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Copyright 1999-2005 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * Portions Copyright 1999-2003 Howard Chu.
  * All rights reserved.
@@ -82,14 +82,8 @@ typedef struct dncookie {
 #endif
 } dncookie;
 
-int ldap_back_freeconn( Operation *op, struct ldapconn *lc );
-struct ldapconn *ldap_back_getconn(struct slap_op *op, struct slap_rep *rs);
-int ldap_back_dobind(struct ldapconn *lc, Operation *op, SlapReply *rs);
-int ldap_back_retry(struct ldapconn *lc, Operation *op, SlapReply *rs);
-int ldap_back_map_result(SlapReply *rs);
-int ldap_back_op_result(struct ldapconn *lc, Operation *op, SlapReply *rs,
-	ber_int_t msgid, int sendok);
-int	back_ldap_LTX_init_module(int argc, char *argv[]);
+#define META_BIND_NRETRIES	3
+#define META_BIND_TIMEOUT	1000
 
 int ldap_back_dn_massage(dncookie *dc, struct berval *dn,
 	struct berval *res);
@@ -103,6 +97,8 @@ int mapping_cmp (const void *, const void *);
 int mapping_dup (void *, void *);
 
 void ldap_back_map_init ( struct ldapmap *lm, struct ldapmapping ** );
+int ldap_back_mapping ( struct ldapmap *map, struct berval *s,
+	struct ldapmapping **m, int remap );
 void ldap_back_map ( struct ldapmap *map, struct berval *s, struct berval *m,
 	int remap );
 #define BACKLDAP_MAP	0
@@ -151,54 +147,48 @@ extern int ldap_dnattr_result_rewrite( dncookie *dc, BerVarray a_vals );
 /* (end of) from back-ldap.h before rwm removal */
 
 struct metasingleconn {
-	int			candidate;
+	int			msc_candidate;
 #define	META_NOT_CANDIDATE	0
 #define	META_CANDIDATE		1
 #define	META_LAST_CONN		-1
 	
-	LDAP            	*ld;
-	struct berval          	bound_dn;
-	struct berval		cred;
-	int             	bound;
+	LDAP            	*msc_ld;
+	struct berval          	msc_bound_ndn;
+	struct berval		msc_cred;
+	int             	msc_bound;
 #define META_UNBOUND		0
 #define META_BOUND		1
 #define META_ANONYMOUS		2
 };
 
-#define META_LAST(lsc)		((lsc)->candidate == META_LAST_CONN)
+#define META_LAST(lsc)		((lsc)->msc_candidate == META_LAST_CONN)
 
 struct metaconn {
-	struct slap_conn	*conn;
-	struct rewrite_info	*rwinfo;
+	struct slap_conn	*mc_conn;
+	struct rewrite_info	*mc_rwinfo;
 	
 	/*
 	 * means that the connection is bound; 
 	 * of course only one target actually is ...
 	 */
-	int             bound_target;
+	int             	mc_bound_target;
 #define META_BOUND_NONE		-1
 #define META_BOUND_ALL		-2
 	/* supersedes the connection stuff */
-	struct metasingleconn *conns;
+	struct metasingleconn	*mc_conns;
 };
 
 struct metatarget {
-	char			*uri;
-	struct berval		psuffix;	/* pretty suffix */
-	struct berval		suffix;		/* normalized suffix */
-	struct berval		binddn;
-	struct berval		bindpw;
+	char			*mt_uri;
+	struct berval		mt_psuffix;		/* pretty suffix */
+	struct berval		mt_nsuffix;		/* normalized suffix */
+	struct berval		mt_binddn;
+	struct berval		mt_bindpw;
 
-	struct berval           pseudorootdn;
-	struct berval           pseudorootpw;
+	struct berval           mt_pseudorootdn;
+	struct berval           mt_pseudorootpw;
 
-#if 0
-	struct rewrite_info	*rwinfo;
-
-	struct ldapmap		oc_map;
-	struct ldapmap		at_map;
-#endif
-	struct ldaprwmap	rwmap;
+	struct ldaprwmap	mt_rwmap;
 };
 
 struct metadncache {
@@ -333,6 +323,7 @@ meta_dncache_dup(
 		void			*c2
 );
 
+#define META_TARGET_NONE	(-1)
 extern int
 meta_dncache_get_target(
 		struct metadncache	*cache,
