@@ -1,8 +1,9 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/delete.c,v 1.15 2004/12/01 23:33:36 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/delete.c,v 1.15.2.1 2005/01/20 18:04:02 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Copyright 1999-2005 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
+ * Portions Copyright 2002 Pierangelo Masarati.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -15,7 +16,8 @@
  */
 /* ACKNOWLEDGEMENTS:
  * This work was initially developed by Dmitry Kovalev for inclusion
- * by OpenLDAP Software.
+ * by OpenLDAP Software.  Additional significant contributors include
+ * Pierangelo Masarati.
  */
 
 #include "portable.h"
@@ -80,34 +82,22 @@ int
 backsql_delete( Operation *op, SlapReply *rs )
 {
 	backsql_info 		*bi = (backsql_info*)op->o_bd->be_private;
-	SQLHDBC 		dbh;
-	SQLHSTMT		sth;
+	SQLHDBC 		dbh = SQL_NULL_HDBC;
+	SQLHSTMT		sth = SQL_NULL_HSTMT;
 	RETCODE			rc;
-	int			retval;
+	int			prc = LDAP_SUCCESS;
 	backsql_oc_map_rec	*oc = NULL;
-	backsql_entryID		e_id = BACKSQL_ENTRYID_INIT;
-	Entry			e;
+	backsql_srch_info	bsi = { 0 };
+	backsql_entryID		e_id = { 0 };
+	Entry			d = { 0 }, p = { 0 }, *e = NULL;
+	struct berval		pdn = BER_BVNULL;
+	int			manageDSAit = get_manageDSAit( op );
 	/* first parameter no */
 	SQLUSMALLINT		pno;
 
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_delete(): deleting entry \"%s\"\n",
 			op->o_req_ndn.bv_val, 0, 0 );
 
-	dnParent( &op->o_req_dn, &e.e_name );
-	dnParent( &op->o_req_ndn, &e.e_nname );
-	e.e_attrs = NULL;
-
-	/* check parent for "children" acl */
-	if ( !access_allowed( op, &e, slap_schema.si_ad_children, 
-			NULL, ACL_WRITE, NULL ) ) {
-		Debug( LDAP_DEBUG_TRACE, "   backsql_delete(): "
-			"no write access to parent\n", 
-			0, 0, 0 );
-		rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
-		goto done;
-
-	}
-	
 	rs->sr_err = backsql_get_db_conn( op, &dbh );
 	if ( rs->sr_err != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "   backsql_delete(): "
@@ -115,40 +105,103 @@ backsql_delete( Operation *op, SlapReply *rs )
 			0, 0, 0 );
 		rs->sr_text = ( rs->sr_err == LDAP_OTHER )
 			? "SQL-backend error" : NULL;
+		e = NULL;
 		goto done;
 	}
 	
-	rs->sr_err = backsql_dn2id( op, rs, &e_id, dbh, &op->o_req_ndn, 1 );
-	if ( rs->sr_err != LDAP_SUCCESS ) {
+	/*
+	 * Get the entry
+	 */
+	bsi.bsi_e = &d;
+	rs->sr_err = backsql_init_search( &bsi, &op->o_req_ndn,
+			LDAP_SCOPE_BASE, 
+			SLAP_NO_LIMIT, SLAP_NO_LIMIT,
+			(time_t)(-1), NULL, dbh, op, rs, slap_anlist_no_attrs,
+			( BACKSQL_ISF_MATCHED | BACKSQL_ISF_GET_ENTRY ) );
+	switch ( rs->sr_err ) {
+	case LDAP_SUCCESS:
+		break;
+
+	case LDAP_REFERRAL:
+		if ( manageDSAit && !BER_BVISNULL( &bsi.bsi_e->e_nname ) &&
+				dn_match( &op->o_req_ndn, &bsi.bsi_e->e_nname ) )
+		{
+			rs->sr_err = LDAP_SUCCESS;
+			rs->sr_text = NULL;
+			rs->sr_matched = NULL;
+			if ( rs->sr_ref ) {
+				ber_bvarray_free( rs->sr_ref );
+				rs->sr_ref = NULL;
+			}
+			break;
+		}
+		e = &d;
+		/* fallthru */
+
+	default:
+		Debug( LDAP_DEBUG_TRACE, "backsql_delete(): "
+			"could not retrieve deleteDN ID - no such entry\n", 
+			0, 0, 0 );
+		if ( !BER_BVISNULL( &d.e_nname ) ) {
+			/* FIXME: should always be true! */
+			e = &d;
+
+		} else {
+			e = NULL;
+		}
+		goto done;
+	}
+
+	if ( get_assert( op ) &&
+			( test_filter( op, &d, get_assertion( op ) )
+			  != LDAP_COMPARE_TRUE ) )
+	{
+		rs->sr_err = LDAP_ASSERTION_FAILED;
+		e = &d;
+		goto done;
+	}
+
+	if ( !access_allowed( op, &d, slap_schema.si_ad_entry, 
+			NULL, ACL_WRITE, NULL ) )
+	{
 		Debug( LDAP_DEBUG_TRACE, "   backsql_delete(): "
-			"could not lookup entry id\n", 0, 0, 0 );
+			"no write access to entry\n", 
+			0, 0, 0 );
+		rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
+		e = &d;
 		goto done;
 	}
 
 	rs->sr_err = backsql_has_children( bi, dbh, &op->o_req_ndn );
 	switch ( rs->sr_err ) {
+	case LDAP_COMPARE_FALSE:
+		rs->sr_err = LDAP_SUCCESS;
+		break;
+
 	case LDAP_COMPARE_TRUE:
+		if ( get_treeDelete( op ) ) {
+			/* not supported yet */ ;
+		}
 		Debug( LDAP_DEBUG_TRACE, "   backsql_delete(): "
 			"entry \"%s\" has children\n",
 			op->o_req_dn.bv_val, 0, 0 );
 		rs->sr_err = LDAP_NOT_ALLOWED_ON_NONLEAF;
 		rs->sr_text = "subtree delete not supported";
-		goto done;
-
-	case LDAP_COMPARE_FALSE:
-		break;
+		/* fallthru */
 
 	default:
+		e = &d;
 		goto done;
 	}
 
-	oc = backsql_id2oc( bi, e_id.eid_oc_id );
+	oc = backsql_id2oc( bi, bsi.bsi_base_id.eid_oc_id );
 	if ( oc == NULL ) {
 		Debug( LDAP_DEBUG_TRACE, "   backsql_delete(): "
 			"cannot determine objectclass of entry -- aborting\n",
 			0, 0, 0 );
 		rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
 		rs->sr_text = "operation not permitted within namingContext";
+		e = NULL;
  		goto done;
 	}
 
@@ -158,12 +211,48 @@ backsql_delete( Operation *op, SlapReply *rs )
 			"for this objectclass - aborting\n", 0, 0, 0 );
 		rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
 		rs->sr_text = "operation not permitted within namingContext";
+		e = NULL;
 		goto done;
 	}
 
+	/*
+	 * Get the parent
+	 */
+	dnParent( &op->o_req_ndn, &pdn );
+	bsi.bsi_e = &p;
+	e_id = bsi.bsi_base_id;
+	rs->sr_err = backsql_init_search( &bsi, &pdn,
+			LDAP_SCOPE_BASE, 
+			SLAP_NO_LIMIT, SLAP_NO_LIMIT,
+			(time_t)(-1), NULL, dbh, op, rs, slap_anlist_no_attrs,
+			BACKSQL_ISF_GET_ENTRY );
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		Debug( LDAP_DEBUG_TRACE, "backsql_delete(): "
+			"could not retrieve deleteDN ID - no such entry\n", 
+			0, 0, 0 );
+		e = &p;
+		goto done;
+	}
+
+	(void)backsql_free_entryID( op, &bsi.bsi_base_id, 0 );
+
+	/* check parent for "children" acl */
+	if ( !access_allowed( op, &p, slap_schema.si_ad_children, 
+			NULL, ACL_WRITE, NULL ) )
+	{
+		Debug( LDAP_DEBUG_TRACE, "   backsql_delete(): "
+			"no write access to parent\n", 
+			0, 0, 0 );
+		rs->sr_err = LDAP_INSUFFICIENT_ACCESS;
+		e = &p;
+		goto done;
+
+	}
+
 	/* avl_apply ... */
 	rs->sr_err = backsql_delete_all_attrs( op, rs, dbh, &e_id, oc );
 	if ( rs->sr_err != LDAP_SUCCESS ) {
+		e = &d;
 		goto done;
 	}
 
@@ -177,12 +266,13 @@ backsql_delete( Operation *op, SlapReply *rs )
 
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "SQL-backend error";
+		e = NULL;
 		goto done;
 	}
 
 	if ( BACKSQL_IS_DEL( oc->bom_expect_return ) ) {
 		pno = 1;
-		rc = backsql_BindParamInt( sth, 1, SQL_PARAM_OUTPUT, &retval );
+		rc = backsql_BindParamInt( sth, 1, SQL_PARAM_OUTPUT, &prc );
 		if ( rc != SQL_SUCCESS ) {
 			Debug( LDAP_DEBUG_TRACE,
 				"   backsql_delete(): "
@@ -194,6 +284,7 @@ backsql_delete( Operation *op, SlapReply *rs )
 
 			rs->sr_text = "SQL-backend error";
 			rs->sr_err = LDAP_OTHER;
+			e = NULL;
 			goto done;
 		}
 
@@ -213,23 +304,38 @@ backsql_delete( Operation *op, SlapReply *rs )
 
 		rs->sr_text = "SQL-backend error";
 		rs->sr_err = LDAP_OTHER;
+		e = NULL;
 		goto done;
 	}
 
 	rc = SQLExecute( sth );
-	if ( rc != SQL_SUCCESS ) {
+	if ( rc == SQL_SUCCESS && prc == LDAP_SUCCESS ) {
+		rs->sr_err = LDAP_SUCCESS;
+
+	} else {
 		Debug( LDAP_DEBUG_TRACE, "   backsql_delete(): "
-			"delete_proc execution failed\n", 0, 0, 0 );
-		backsql_PrintErrors( bi->sql_db_env, dbh, sth, rc );
+			"delete_proc execution failed (rc=%d, prc=%d)\n",
+			rc, prc, 0 );
+
+
+		if ( prc != LDAP_SUCCESS ) {
+			/* SQL procedure executed fine 
+			 * but returned an error */
+			rs->sr_err = BACKSQL_SANITIZE_ERROR( prc );
+
+		} else {
+			backsql_PrintErrors( bi->sql_db_env, dbh,
+					sth, rc );
+			rs->sr_err = LDAP_OTHER;
+		}
 		SQLFreeStmt( sth, SQL_DROP );
-		rs->sr_err = LDAP_OTHER;
-		rs->sr_text = "SQL-backend error";
+		e = &d;
 		goto done;
 	}
 	SQLFreeStmt( sth, SQL_DROP );
 
 	/* delete "auxiliary" objectClasses, if any... */
-	rc = backsql_Prepare( dbh, &sth, bi->sql_delobjclasses_query, 0 );
+	rc = backsql_Prepare( dbh, &sth, bi->sql_delobjclasses_stmt, 0 );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE,
 			"   backsql_delete(): "
@@ -239,6 +345,7 @@ backsql_delete( Operation *op, SlapReply *rs )
 
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "SQL-backend error";
+		e = NULL;
 		goto done;
 	}
 
@@ -255,6 +362,7 @@ backsql_delete( Operation *op, SlapReply *rs )
 
 		rs->sr_text = "SQL-backend error";
 		rs->sr_err = LDAP_OTHER;
+		e = NULL;
 		goto done;
 	}
 
@@ -274,62 +382,13 @@ backsql_delete( Operation *op, SlapReply *rs )
 		SQLFreeStmt( sth, SQL_DROP );
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "SQL-backend error";
-		goto done;
-	}
-	SQLFreeStmt( sth, SQL_DROP );
-
-	/* delete referrals, if any... */
-	rc = backsql_Prepare( dbh, &sth, bi->sql_delreferrals_query, 0 );
-	if ( rc != SQL_SUCCESS ) {
-		Debug( LDAP_DEBUG_TRACE,
-			"   backsql_delete(): "
-			"error preparing ldap_referrals delete query\n", 
-			0, 0, 0 );
-		backsql_PrintErrors( bi->sql_db_env, dbh, sth, rc );
-
-		rs->sr_err = LDAP_OTHER;
-		rs->sr_text = "SQL-backend error";
-		goto done;
-	}
-
-	rc = backsql_BindParamID( sth, 1, SQL_PARAM_INPUT, &e_id.eid_id );
-	if ( rc != SQL_SUCCESS ) {
-		Debug( LDAP_DEBUG_TRACE,
-			"   backsql_delete(): "
-			"error binding referrals entry ID parameter "
-			"for objectClass %s\n",
-			oc->bom_oc->soc_cname.bv_val, 0, 0 );
-		backsql_PrintErrors( bi->sql_db_env, dbh, 
-			sth, rc );
-		SQLFreeStmt( sth, SQL_DROP );
-
-		rs->sr_text = "SQL-backend error";
-		rs->sr_err = LDAP_OTHER;
-		goto done;
-	}
-
-	rc = SQLExecute( sth );
-	switch ( rc ) {
-	case SQL_NO_DATA:
-		/* apparently there were no referrals
-		 * for this entry... */
-	case SQL_SUCCESS:
-		break;
-
-	default:
-		Debug( LDAP_DEBUG_TRACE, "   backsql_delete(): "
-			"failed to delete record from ldap_referrals\n", 
-			0, 0, 0 );
-		backsql_PrintErrors( bi->sql_db_env, dbh, sth, rc );
-		SQLFreeStmt( sth, SQL_DROP );
-		rs->sr_err = LDAP_OTHER;
-		rs->sr_text = "SQL-backend error";
+		e = NULL;
 		goto done;
 	}
 	SQLFreeStmt( sth, SQL_DROP );
 
 	/* delete entry... */
-	rc = backsql_Prepare( dbh, &sth, bi->sql_delentry_query, 0 );
+	rc = backsql_Prepare( dbh, &sth, bi->sql_delentry_stmt, 0 );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE,
 			"   backsql_delete(): "
@@ -339,6 +398,7 @@ backsql_delete( Operation *op, SlapReply *rs )
 
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "SQL-backend error";
+		e = NULL;
 		goto done;
 	}
 
@@ -355,6 +415,7 @@ backsql_delete( Operation *op, SlapReply *rs )
 
 		rs->sr_text = "SQL-backend error";
 		rs->sr_err = LDAP_OTHER;
+		e = NULL;
 		goto done;
 	}
 
@@ -367,29 +428,59 @@ backsql_delete( Operation *op, SlapReply *rs )
 		SQLFreeStmt( sth, SQL_DROP );
 		rs->sr_err = LDAP_OTHER;
 		rs->sr_text = "SQL-backend error";
+		e = NULL;
 		goto done;
 	}
 	SQLFreeStmt( sth, SQL_DROP );
 
-	/*
-	 * Commit only if all operations succeed
-	 *
-	 * FIXME: backsql_add() does not fail if add operations 
-	 * are not available for some attributes, or if
-	 * a multiple value add actually results in a replace, 
-	 * or if a single operation on an attribute fails 
-	 * for any reason
-	 */
-	SQLTransact( SQL_NULL_HENV, dbh, 
-			op->o_noop ? SQL_ROLLBACK : SQL_COMMIT );
-
 	rs->sr_err = LDAP_SUCCESS;
 
+	/*
+	 * Commit only if all operations succeed
+	 */
+	if ( sth != SQL_NULL_HSTMT ) {
+		SQLUSMALLINT	CompletionType = SQL_ROLLBACK;
+	
+		if ( rs->sr_err == LDAP_SUCCESS && !op->o_noop ) {
+			CompletionType = SQL_COMMIT;
+		}
+
+		SQLTransact( SQL_NULL_HENV, dbh, CompletionType );
+	}
+
 done:;
+#ifdef SLAP_ACL_HONOR_DISCLOSE
+	if ( e != NULL ) {
+		if ( !access_allowed( op, e, slap_schema.si_ad_entry, NULL,
+					ACL_DISCLOSE, NULL ) )
+		{
+			rs->sr_err = LDAP_NO_SUCH_OBJECT;
+			rs->sr_text = NULL;
+			rs->sr_matched = NULL;
+			if ( rs->sr_ref ) {
+				ber_bvarray_free( rs->sr_ref );
+				rs->sr_ref = NULL;
+			}
+		}
+	}
+#endif /* SLAP_ACL_HONOR_DISCLOSE */
+
 	send_ldap_result( op, rs );
 
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_delete()\n", 0, 0, 0 );
 
-	return ( ( rs->sr_err == LDAP_SUCCESS ) ? op->o_noop : 1 );
+	if ( !BER_BVISNULL( &e_id.eid_ndn ) ) {
+		(void)backsql_free_entryID( op, &e_id, 0 );
+	}
+
+	if ( !BER_BVISNULL( &d.e_nname ) ) {
+		entry_clean( &d );
+	}
+
+	if ( !BER_BVISNULL( &p.e_nname ) ) {
+		entry_clean( &p );
+	}
+
+	return rs->sr_err;
 }
 
