@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.318 2004/12/08 02:47:48 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.318.2.2 2005/01/24 21:28:51 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -1714,10 +1714,12 @@ slapd_daemon_task(
 
 		case 0:		/* timeout - let threads run */
 			ebadf = 0;
+#ifndef HAVE_YIELDING_SELECT
 			Debug( LDAP_DEBUG_CONNS, "daemon: select timeout - yielding\n",
 			    0, 0, 0 );
 
 			ldap_pvt_thread_yield();
+#endif
 			continue;
 
 		default:	/* something happened - deal with it */
@@ -1770,7 +1772,9 @@ slapd_daemon_task(
 
 		/* bypass the following tests if no descriptors left */
 		if ( ns <= 0 ) {
+#ifndef HAVE_YIELDING_SELECT
 			ldap_pvt_thread_yield();
+#endif
 			continue;
 		}
 
@@ -1844,6 +1848,7 @@ slapd_daemon_task(
 				}
 				slapd_close( wd );
 			}
+			SLAP_EVENT_CLR_WRITE( wd );
 		}
 
 		for ( i = 0; nrfds > 0; i++ )
@@ -1965,7 +1970,9 @@ slapd_daemon_task(
 		}
 #endif	/* SLAP_EVENTS_ARE_INDEXED */
 
+#ifndef HAVE_YIELDING_SELECT
 		ldap_pvt_thread_yield();
+#endif
 	}
 
 	if( slapd_shutdown == 1 ) {
