@@ -1,8 +1,8 @@
 /* refint.c - referential integrity module */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/refint.c,v 1.7 2004/11/25 20:16:56 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/refint.c,v 1.7.2.2 2005/02/02 17:32:45 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2004 The OpenLDAP Foundation.
+ * Copyright 2004-2005 The OpenLDAP Foundation.
  * Portions Copyright 2004 Symas Corporation.
  * All rights reserved.
  *
@@ -551,6 +551,7 @@ refint_response(
 	nop.o_tag	= LDAP_REQ_SEARCH;
 	nop.ors_scope	= LDAP_SCOPE_SUBTREE;
 	nop.ors_deref	= LDAP_DEREF_NEVER;
+	nop.ors_limit   = NULL;
 	nop.ors_slimit	= SLAP_NO_LIMIT;
 	nop.ors_tlimit	= SLAP_NO_LIMIT;
 
