@@ -1,8 +1,8 @@
 /* rwm.c - rewrite/remap operations */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwm.c,v 1.37.2.1 2004/12/30 21:29:58 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwm.c,v 1.37.2.4 2005/02/02 17:32:45 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2004 The OpenLDAP Foundation.
+ * Copyright 2003-2005 The OpenLDAP Foundation.
  * Portions Copyright 2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -34,7 +34,6 @@ rwm_op_dn_massage( Operation *op, SlapReply *rs, void *cookie )
 			(struct ldaprwmap *)on->on_bi.bi_private;
 
 	struct berval		dn = BER_BVNULL,
-				*dnp = NULL,
 				ndn = BER_BVNULL;
 	int			rc = 0;
 	dncookie		dc;
@@ -56,28 +55,32 @@ rwm_op_dn_massage( Operation *op, SlapReply *rs, void *cookie )
 	 * and the caller sets op->o_req_dn = op->o_req_ndn,
 	 * only rewrite the op->o_req_ndn and use it as 
 	 * op->o_req_dn as well */
+	ndn = op->o_req_ndn;
 	if ( op->o_req_dn.bv_val != op->o_req_ndn.bv_val ) {
-		dnp = &dn;
+		dn = op->o_req_dn;
+		rc = rwm_dn_massage_pretty_normalize( &dc, &op->o_req_dn, &dn, &ndn );
+	} else {
+		rc = rwm_dn_massage_normalize( &dc, &op->o_req_ndn, &ndn );
 	}
 
-	rc = rwm_dn_massage( &dc, &op->o_req_dn, dnp, &ndn );
 	if ( rc != LDAP_SUCCESS ) {
 		return rc;
 	}
 
-	if ( ( dnp && dn.bv_val == op->o_req_dn.bv_val ) ||
-		( !dnp && ndn.bv_val == op->o_req_ndn.bv_val ) ) {
+	if ( ( op->o_req_dn.bv_val != op->o_req_ndn.bv_val && dn.bv_val == op->o_req_dn.bv_val )
+			|| ndn.bv_val == op->o_req_ndn.bv_val )
+	{
 		return LDAP_SUCCESS;
 	}
 
 	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
-	if ( dnp ) {
+	op->o_req_ndn = ndn;
+	if ( op->o_req_dn.bv_val != op->o_req_ndn.bv_val ) {
 		op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
 		op->o_req_dn = dn;
 	} else {
 		op->o_req_dn = ndn;
 	}
-	op->o_req_ndn = ndn;
 
 	return LDAP_SUCCESS;
 }
@@ -93,6 +96,7 @@ rwm_op_add( Operation *op, SlapReply *rs )
 				i;
 	Attribute		**ap = NULL;
 	char			*olddn = op->o_req_dn.bv_val;
+	int			isupdate;
 
 #ifdef ENABLE_REWRITE
 	rc = rwm_op_dn_massage( op, rs, "addDN" );
@@ -115,56 +119,99 @@ rwm_op_add( Operation *op, SlapReply *rs )
 	}
 
 	/* Count number of attributes in entry */ 
+	isupdate = be_shadow_update( op );
 	for ( i = 0, ap = &op->oq_add.rs_e->e_attrs; *ap; ) {
-		struct berval	mapped;
 		Attribute	*a;
 
-		if ( (*ap)->a_desc->ad_type->sat_no_user_mod ) {
-			goto next_attr;
-		}
-
-		rwm_map( &rwmap->rwm_at, &(*ap)->a_desc->ad_cname,
-				&mapped, RWM_MAP );
-		if ( BER_BVISNULL( &mapped ) || BER_BVISEMPTY( &mapped ) ) {
-			goto cleanup_attr;
-		}
-
-		if ( (*ap)->a_desc->ad_type->sat_syntax
-				== slap_schema.si_syn_distinguishedName )
+		if ( (*ap)->a_desc == slap_schema.si_ad_objectClass ||
+				(*ap)->a_desc == slap_schema.si_ad_structuralObjectClass )
 		{
-			/*
-			 * FIXME: rewrite could fail; in this case
-			 * the operation should give up, right?
-			 */
-#ifdef ENABLE_REWRITE
-			rc = rwm_dnattr_rewrite( op, rs, "addAttrDN",
-					(*ap)->a_vals,
-					(*ap)->a_nvals ? &(*ap)->a_nvals : NULL );
-#else /* ! ENABLE_REWRITE */
-			rc = 1;
-			rc = rwm_dnattr_rewrite( op, rs, &rc, (*ap)->a_vals,
-					(*ap)->a_nvals ? &(*ap)->a_nvals : NULL );
-#endif /* ! ENABLE_REWRITE */
-			if ( rc ) {
-				goto cleanup_attr;
+			int		j, last;
+
+			for ( last = 0; !BER_BVISNULL( &(*ap)->a_vals[ last ] ); last++ )
+					/* count values */ ;
+			last--;
+			for ( j = 0; !BER_BVISNULL( &(*ap)->a_vals[ j ] ); j++ ) {
+				struct ldapmapping	*mapping = NULL;
+
+				( void )rwm_mapping( &rwmap->rwm_oc, &(*ap)->a_vals[ j ],
+						&mapping, RWM_MAP );
+				if ( mapping == NULL ) {
+					if ( rwmap->rwm_at.drop_missing ) {
+						/* FIXME: we allow to remove objectClasses as well;
+						 * if the resulting entry is inconsistent, that's
+						 * the relayed database's business...
+						 */
+						ch_free( (*ap)->a_vals[ j ].bv_val );
+						if ( last > j ) {
+							(*ap)->a_vals[ j ] = (*ap)->a_vals[ last ];
+						}
+						BER_BVZERO( &(*ap)->a_vals[ last ] );
+						last--;
+						j--;
+					}
+
+				} else {
+					ch_free( (*ap)->a_vals[ j ].bv_val );
+					ber_dupbv( &(*ap)->a_vals[ j ], &mapping->m_dst );
+				}
 			}
 
-		} else if ( (*ap)->a_desc == slap_schema.si_ad_ref ) {
+		} else if ( !isupdate && (*ap)->a_desc->ad_type->sat_no_user_mod ) {
+			goto next_attr;
+
+		} else {
+			struct ldapmapping	*mapping = NULL;
+
+			( void )rwm_mapping( &rwmap->rwm_at, &(*ap)->a_desc->ad_cname,
+					&mapping, RWM_MAP );
+			if ( mapping == NULL ) {
+				if ( rwmap->rwm_at.drop_missing ) {
+					goto cleanup_attr;
+				}
+			}
+
+			if ( (*ap)->a_desc->ad_type->sat_syntax
+					== slap_schema.si_syn_distinguishedName )
+			{
+				/*
+				 * FIXME: rewrite could fail; in this case
+				 * the operation should give up, right?
+				 */
 #ifdef ENABLE_REWRITE
-			rc = rwm_referral_rewrite( op, rs, "referralAttrDN",
-					(*ap)->a_vals,
-					(*ap)->a_nvals ? &(*ap)->a_nvals : NULL );
+				rc = rwm_dnattr_rewrite( op, rs, "addAttrDN",
+						(*ap)->a_vals,
+						(*ap)->a_nvals ? &(*ap)->a_nvals : NULL );
 #else /* ! ENABLE_REWRITE */
-			rc = 1;
-			rc = rwm_referral_rewrite( op, rs, &rc, (*ap)->a_vals,
-					(*ap)->a_nvals ? &(*ap)->a_nvals : NULL );
+				rc = 1;
+				rc = rwm_dnattr_rewrite( op, rs, &rc, (*ap)->a_vals,
+						(*ap)->a_nvals ? &(*ap)->a_nvals : NULL );
 #endif /* ! ENABLE_REWRITE */
-			if ( rc != LDAP_SUCCESS ) {
-				goto cleanup_attr;
+				if ( rc ) {
+					goto cleanup_attr;
+				}
+
+			} else if ( (*ap)->a_desc == slap_schema.si_ad_ref ) {
+#ifdef ENABLE_REWRITE
+				rc = rwm_referral_rewrite( op, rs, "referralAttrDN",
+						(*ap)->a_vals,
+						(*ap)->a_nvals ? &(*ap)->a_nvals : NULL );
+#else /* ! ENABLE_REWRITE */
+				rc = 1;
+				rc = rwm_referral_rewrite( op, rs, &rc, (*ap)->a_vals,
+						(*ap)->a_nvals ? &(*ap)->a_nvals : NULL );
+#endif /* ! ENABLE_REWRITE */
+				if ( rc != LDAP_SUCCESS ) {
+					goto cleanup_attr;
+				}
+			}
+		
+			if ( mapping != NULL ) {
+				assert( mapping->m_dst_ad );
+				(*ap)->a_desc = mapping->m_dst_ad;
 			}
 		}
 
-
 next_attr:;
 		ap = &(*ap)->a_next;
 		continue;
@@ -276,6 +323,7 @@ rwm_op_compare( Operation *op, SlapReply *rs )
 			}
 
 		} else {
+			assert( mapping->m_dst_ad );
 			ad = mapping->m_dst_ad;
 		}
 
@@ -337,6 +385,7 @@ rwm_op_modify( Operation *op, SlapReply *rs )
 	struct ldaprwmap	*rwmap = 
 			(struct ldaprwmap *)on->on_bi.bi_private;
 
+	int			isupdate;
 	Modifications		**mlp;
 	int			rc;
 
@@ -352,66 +401,63 @@ rwm_op_modify( Operation *op, SlapReply *rs )
 		return -1;
 	}
 
+	isupdate = be_shadow_update( op );
 	for ( mlp = &op->oq_modify.rs_modlist; *mlp; ) {
-		int		is_oc = 0;
-		Modifications	*ml;
-
-		if ( (*mlp)->sml_desc->ad_type->sat_no_user_mod  ) {
-			goto next_mod;
-		}
+		int			is_oc = 0;
+		Modifications		*ml;
+		struct ldapmapping	*mapping = NULL;
 
 		if ( (*mlp)->sml_desc == slap_schema.si_ad_objectClass 
-				|| (*mlp)->sml_desc == slap_schema.si_ad_structuralObjectClass ) {
+				|| (*mlp)->sml_desc == slap_schema.si_ad_structuralObjectClass )
+		{
 			is_oc = 1;
 
+		} else if ( !isupdate && (*mlp)->sml_desc->ad_type->sat_no_user_mod  ) {
+			goto next_mod;
+
 		} else {
-			struct ldapmapping	*m;
 			int			drop_missing;
 
-			drop_missing = rwm_mapping( &rwmap->rwm_at, &(*mlp)->sml_desc->ad_cname, &m, RWM_MAP );
-			if ( drop_missing || ( m != NULL && BER_BVISNULL( &m->m_dst ) ) )
+			drop_missing = rwm_mapping( &rwmap->rwm_at,
+					&(*mlp)->sml_desc->ad_cname,
+					&mapping, RWM_MAP );
+			if ( drop_missing || ( mapping != NULL && BER_BVISNULL( &mapping->m_dst ) ) )
 			{
 				goto cleanup_mod;
 			}
-
-			if ( m ) {
-				/* use new attribute description */
-				assert( m->m_dst_ad );
-				(*mlp)->sml_desc = m->m_dst_ad;
-			}
 		}
 
 		if ( (*mlp)->sml_values != NULL ) {
 			if ( is_oc ) {
 				int	last, j;
 
-				for ( last = 0; !BER_BVISNULL( &(*mlp)->sml_values[last] ); last++ )
+				for ( last = 0; !BER_BVISNULL( &(*mlp)->sml_values[ last ] ); last++ )
 					/* count values */ ;
 				last--;
 
-				for ( j = 0; !BER_BVISNULL( &(*mlp)->sml_values[j] ); j++ ) {
-					struct berval	mapped = BER_BVNULL;
+				for ( j = 0; !BER_BVISNULL( &(*mlp)->sml_values[ j ] ); j++ ) {
+					struct ldapmapping	*mapping = NULL;
 
-					rwm_map( &rwmap->rwm_oc,
-							&(*mlp)->sml_values[j],
-							&mapped, RWM_MAP );
-					if ( BER_BVISNULL( &mapped ) || BER_BVISEMPTY( &mapped ) ) {
-						/* FIXME: we allow to remove objectClasses as well;
-						 * if the resulting entry is inconsistent, that's
-						 * the relayed database's business...
-						 */
-#if 0
-						goto cleanup_mod;
-#endif
-						if ( last > j ) {
-							(*mlp)->sml_values[j] = (*mlp)->sml_values[last];
-							BER_BVZERO( &(*mlp)->sml_values[last] );
+					( void )rwm_mapping( &rwmap->rwm_oc, &(*mlp)->sml_values[ j ],
+							&mapping, RWM_MAP );
+					if ( mapping == NULL ) {
+						if ( rwmap->rwm_at.drop_missing ) {
+							/* FIXME: we allow to remove objectClasses as well;
+							 * if the resulting entry is inconsistent, that's
+							 * the relayed database's business...
+							 */
+							ch_free( (*mlp)->sml_values[ j ].bv_val );
+							if ( last > j ) {
+								(*mlp)->sml_values[ j ] = (*mlp)->sml_values[ last ];
+							}
+							BER_BVZERO( &(*mlp)->sml_values[ last ] );
+							last--;
+							j--;
 						}
-						last--;
-
+	
 					} else {
-						ch_free( (*mlp)->sml_values[j].bv_val );
-						ber_dupbv( &(*mlp)->sml_values[j], &mapped );
+						ch_free( (*mlp)->sml_values[ j ].bv_val );
+						ber_dupbv( &(*mlp)->sml_values[ j ], &mapping->m_dst );
 					}
 				}
 
@@ -454,6 +500,12 @@ rwm_op_modify( Operation *op, SlapReply *rs )
 		}
 
 next_mod:;
+		if ( mapping != NULL ) {
+			/* use new attribute description */
+			assert( mapping->m_dst_ad );
+			(*mlp)->sml_desc = mapping->m_dst_ad;
+		}
+
 		mlp = &(*mlp)->sml_next;
 		continue;
 
@@ -494,7 +546,9 @@ rwm_op_modrdn( Operation *op, SlapReply *rs )
 		dc.tofrom = 0;
 		dc.normalized = 0;
 #endif /* ! ENABLE_REWRITE */
-		rc = rwm_dn_massage( &dc, op->orr_newSup, &newSup, &nnewSup );
+		newSup = *op->orr_newSup;
+		nnewSup = *op->orr_nnewSup;
+		rc = rwm_dn_massage_pretty_normalize( &dc, op->orr_newSup, &newSup, &nnewSup );
 		if ( rc != LDAP_SUCCESS ) {
 			op->o_bd->bd_info = (BackendInfo *)on->on_info;
 			send_ldap_error( op, rs, rc, "newSuperiorDN massage error" );
@@ -709,7 +763,8 @@ rwm_matched( Operation *op, SlapReply *rs )
 	dc.normalized = 0;
 #endif /* ! ENABLE_REWRITE */
 	ber_str2bv( rs->sr_matched, 0, 0, &dn );
-	rc = rwm_dn_massage( &dc, &dn, &mdn, NULL );
+	mdn = dn;
+	rc = rwm_dn_massage_pretty( &dc, &dn, &mdn );
 	if ( rc != LDAP_SUCCESS ) {
 		rs->sr_err = rc;
 		rs->sr_text = "Rewrite error";
@@ -730,7 +785,7 @@ rwm_matched( Operation *op, SlapReply *rs )
 }
 
 static int
-rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first )
+rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first, int stripEntryDN )
 {
 	slap_overinst		*on = (slap_overinst *) op->o_bd->bd_info;
 	struct ldaprwmap	*rwmap = 
@@ -739,6 +794,7 @@ rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first )
 	dncookie		dc;
 	int			rc;
 	Attribute		**ap;
+	int			isupdate;
 
 	/*
 	 * Rewrite the dn attrs, if needed
@@ -764,8 +820,9 @@ rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first )
 	 * an error (because multiple instances of attrs in 
 	 * response are not valid), or merge the values (what
 	 * about duplicate values?) */
+	isupdate = be_shadow_update( op );
 	for ( ap = a_first; *ap; ) {
-		struct ldapmapping	*m;
+		struct ldapmapping	*mapping;
 		int			drop_missing;
 		int			last;
 		Attribute		*a;
@@ -774,25 +831,39 @@ rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first )
 		{
 			/* go on */ ;
 			
-		} else if ( op->ors_attrs != NULL && 
-				!SLAP_USERATTRS( rs->sr_attr_flags ) && 
-				!ad_inlist( (*ap)->a_desc, op->ors_attrs ) )
-		{
-			goto cleanup_attr;
+		} else {
+			drop_missing = rwm_mapping( &rwmap->rwm_at,
+					&(*ap)->a_desc->ad_cname, &mapping, RWM_REMAP );
+			if ( drop_missing || ( mapping != NULL && BER_BVISEMPTY( &mapping->m_dst ) ) )
+			{
+				goto cleanup_attr;
+			}
+
+			if ( mapping != NULL ) {
+				(*ap)->a_desc = mapping->m_dst_ad;
+			}
+
+			if ( op->ors_attrs != NULL && 
+					!SLAP_USERATTRS( rs->sr_attr_flags ) &&
+					!ad_inlist( (*ap)->a_desc, op->ors_attrs ) )
+			{
+				goto cleanup_attr;
+			}
 		}
 
-		if ( (*ap)->a_desc->ad_type->sat_no_user_mod 
+		if ( (*ap)->a_desc == slap_schema.si_ad_entryDN ) {
+			if ( stripEntryDN ) {
+				/* will be generated by frontend */
+				goto cleanup_attr;
+			}
+			
+		} else if ( !isupdate
+			&& (*ap)->a_desc->ad_type->sat_no_user_mod 
 			&& (*ap)->a_desc->ad_type != slap_schema.si_at_undefined )
 		{
 			goto next_attr;
 		}
 
-		drop_missing = rwm_mapping( &rwmap->rwm_at,
-				&(*ap)->a_desc->ad_cname, &m, RWM_REMAP );
-		if ( drop_missing || ( m != NULL && BER_BVISEMPTY( &m->m_dst ) ) ) {
-			goto cleanup_attr;
-		}
-
 		for ( last = 0; !BER_BVISNULL( &(*ap)->a_vals[last] ); last++ )
 			/* just count */ ;
 
@@ -865,10 +936,10 @@ rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first )
 			}
 		}
 
-		if ( m != NULL ) {
+		if ( mapping != NULL ) {
 			/* rewrite the attribute description */
-			assert( m->m_dst_ad );
-			(*ap)->a_desc = m->m_dst_ad;
+			assert( mapping->m_dst_ad );
+			(*ap)->a_desc = mapping->m_dst_ad;
 		}
 
 next_attr:;
@@ -938,7 +1009,9 @@ rwm_send_entry( Operation *op, SlapReply *rs )
 	 * from the one known to the meta, and a DN with unknown
 	 * attributes is returned.
 	 */
-	rc = rwm_dn_massage( &dc, &e->e_name, &dn, &ndn );
+	dn = e->e_name;
+	ndn = e->e_nname;
+	rc = rwm_dn_massage_pretty_normalize( &dc, &e->e_name, &dn, &ndn );
 	if ( rc != LDAP_SUCCESS ) {
 		rc = 1;
 		goto fail;
@@ -958,7 +1031,13 @@ rwm_send_entry( Operation *op, SlapReply *rs )
 	/* FIXME: the entries are in the remote mapping form;
 	 * so we need to select those attributes we are willing
 	 * to return, and remap them accordingly */
-	(void)rwm_attrs( op, rs, &e->e_attrs );
+	(void)rwm_attrs( op, rs, &e->e_attrs, 1 );
+
+#if 0
+	if ( rs->sr_operational_attrs ) {
+		(void)rwm_attrs( op, rs, &rs->sr_operational_attrs, 0 );
+	}
+#endif
 
 	rs->sr_entry = e;
 	rs->sr_flags = flags;
@@ -988,7 +1067,7 @@ rwm_operational( Operation *op, SlapReply *rs )
 	 * so we need to select those attributes we are willing
 	 * to return, and remap them accordingly */
 	if ( rs->sr_operational_attrs ) {
-		rwm_attrs( op, rs, &rs->sr_operational_attrs );
+		rwm_attrs( op, rs, &rs->sr_operational_attrs, 1 );
 	}
 
 	return SLAP_CB_CONTINUE;
