@@ -1,8 +1,8 @@
 /* result.c - routines to send ldap results, errors, and referrals */
-/* $OpenLDAP: pkg/ldap/servers/slapd/result.c,v 1.244 2004/11/10 20:38:41 jongchoi Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/result.c,v 1.244.2.2 2005/02/02 17:32:44 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -453,7 +453,7 @@ send_ldap_response(
 
 	ldap_pvt_thread_mutex_lock( &slap_counters.sc_sent_mutex );
 	ldap_pvt_mp_add_ulong( slap_counters.sc_pdu, 1 );
-	ldap_pvt_mp_add_ulong( slap_counters.sc_bytes, bytes );
+	ldap_pvt_mp_add_ulong( slap_counters.sc_bytes, (unsigned long)bytes );
 	ldap_pvt_thread_mutex_unlock( &slap_counters.sc_sent_mutex );
 
 cleanup:;
@@ -1180,7 +1180,7 @@ slap_send_search_entry( Operation *op, SlapReply *rs )
 		rs->sr_nentries++;
 
 		ldap_pvt_thread_mutex_lock( &slap_counters.sc_sent_mutex );
-		ldap_pvt_mp_add_ulong( slap_counters.sc_bytes, bytes );
+		ldap_pvt_mp_add_ulong( slap_counters.sc_bytes, (unsigned long)bytes );
 		ldap_pvt_mp_add_ulong( slap_counters.sc_entries, 1 );
 		ldap_pvt_mp_add_ulong( slap_counters.sc_pdu, 1 );
 		ldap_pvt_thread_mutex_unlock( &slap_counters.sc_sent_mutex );
@@ -1369,7 +1369,7 @@ slap_send_search_reference( Operation *op, SlapReply *rs )
 	ber_free_buf( ber );
 
 	ldap_pvt_thread_mutex_lock( &slap_counters.sc_sent_mutex );
-	ldap_pvt_mp_add_ulong( slap_counters.sc_bytes, bytes );
+	ldap_pvt_mp_add_ulong( slap_counters.sc_bytes, (unsigned long)bytes );
 	ldap_pvt_mp_add_ulong( slap_counters.sc_refs, 1 );
 	ldap_pvt_mp_add_ulong( slap_counters.sc_pdu, 1 );
 	ldap_pvt_thread_mutex_unlock( &slap_counters.sc_sent_mutex );
