@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/saslauthz.c,v 1.144 2004/12/08 18:09:54 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/saslauthz.c,v 1.144.2.2 2005/01/24 21:28:51 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * Portions Copyright 2000 Mark Adamson, Carnegie Mellon.
  * All rights reserved.
  *
@@ -700,7 +700,9 @@ static int slap_authz_regexp( struct berval *in, struct berval *out,
 		if ( !BER_BVISNULL( out ) ) {
 			char *val = out->bv_val;
 			ber_str2bv_x( val, 0, 1, out, ctx );
-			free( val );
+			if ( val != in->bv_val ) {
+				free( val );
+			}
 		} else {
 			ber_dupbv_x( out, in, ctx );
 		}
@@ -1099,8 +1101,7 @@ void slap_sasl2dn( Operation *opx,
 		"converting SASL name %s to a DN\n",
 		saslname->bv_val, 0,0 );
 
-	sasldn->bv_val = NULL;
-	sasldn->bv_len = 0;
+	BER_BVZERO( sasldn );
 	cb.sc_private = sasldn;
 
 	/* Convert the SASL name into a minimal URI */
