@@ -1,8 +1,8 @@
 /* schema_check.c - routines to enforce schema definitions */
-/* $OpenLDAP: pkg/ldap/servers/slapd/schema_check.c,v 1.94 2004/10/06 03:51:39 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/schema_check.c,v 1.94.2.1 2005/01/20 18:03:56 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -684,6 +684,8 @@ int structural_class(
 		return LDAP_OBJECT_CLASS_VIOLATION;
 	}
 
+	*text = NULL;
+
 	return LDAP_SUCCESS;
 }
 
