@@ -1,8 +1,8 @@
 /* slap.h - stand alone ldap server include file */
-/* $OpenLDAP: pkg/ldap/servers/slapd/slap.h,v 1.612.2.1 2004/12/30 21:29:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slap.h,v 1.612.2.5 2005/02/02 17:32:44 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -58,6 +58,21 @@
 
 LDAP_BEGIN_DECL
 
+#ifdef LDAP_DEVEL
+#define SLAP_ACL_HONOR_DISCLOSE	/* partially implemented */
+#define SLAP_DYNACL
+#define LDAP_COMP_MATCH			/* experimental */
+#define LDAP_DYNAMIC_OBJECTS
+#define LDAP_SYNC_TIMESTAMP
+#define LDAP_COLLECTIVE_ATTRIBUTES
+#define SLAP_CONTROL_X_TREE_DELETE LDAP_CONTROL_X_TREE_DELETE
+#endif
+
+#if defined(LDAP_DEVEL) && defined(ENABLE_REWRITE)
+/* use librewrite for sasl-regexp */
+#define SLAP_AUTH_REWRITE	1
+#endif /* LDAP_DEVEL && ENABLE_REWRITE */
+
 /*
  * SLAPD Memory allocation macros
  *
@@ -324,10 +339,6 @@ typedef int slap_syntax_transform_func LDAP_P((
 	struct berval * out,
 	void *memctx));
 
-#ifdef LDAP_DEVEL
-#define LDAP_COMP_MATCH
-#endif
-
 #ifdef LDAP_COMP_MATCH
 typedef void* slap_component_transform_func LDAP_P((
 	struct berval * in ));
@@ -1123,11 +1134,13 @@ typedef struct slap_ldap_modlist {
 typedef enum slap_access_e {
 	ACL_INVALID_ACCESS = -1,
 	ACL_NONE = 0,
+	ACL_DISCLOSE,
 	ACL_AUTH,
 	ACL_COMPARE,
 	ACL_SEARCH,
 	ACL_READ,
-	ACL_WRITE
+	ACL_WRITE,
+	ACL_MANAGE
 } slap_access_t;
 
 typedef enum slap_control_e {
@@ -1166,10 +1179,6 @@ typedef struct slap_authz_info {
 } AuthorizationInformation;
 
 
-#ifdef LDAP_DEVEL
-#define SLAP_DYNACL
-#endif /* LDAP_DEVEL */
-
 #ifdef SLAP_DYNACL
 struct slap_op;
 
@@ -1209,11 +1218,13 @@ typedef struct slap_access {
 #define ACL_ACCESS2PRIV(access)	(0x01U << (access))
 
 #define ACL_PRIV_NONE			ACL_ACCESS2PRIV( ACL_NONE )
+#define ACL_PRIV_DISCLOSE		ACL_ACCESS2PRIV( ACL_DISCLOSE )
 #define ACL_PRIV_AUTH			ACL_ACCESS2PRIV( ACL_AUTH )
 #define ACL_PRIV_COMPARE		ACL_ACCESS2PRIV( ACL_COMPARE )
 #define ACL_PRIV_SEARCH			ACL_ACCESS2PRIV( ACL_SEARCH )
 #define ACL_PRIV_READ			ACL_ACCESS2PRIV( ACL_READ )
 #define ACL_PRIV_WRITE			ACL_ACCESS2PRIV( ACL_WRITE )
+#define ACL_PRIV_MANAGE			ACL_ACCESS2PRIV( ACL_MANAGE )
 
 #define ACL_PRIV_MASK			0x00ffUL
 
@@ -1242,26 +1253,32 @@ typedef struct slap_access {
 #define ACL_IS_SUBTRACTIVE(m)	ACL_PRIV_ISSET((m),ACL_PRIV_SUBSTRACTIVE)
 
 #define ACL_LVL_NONE			(ACL_PRIV_NONE|ACL_PRIV_LEVEL)
-#define ACL_LVL_AUTH			(ACL_PRIV_AUTH|ACL_LVL_NONE)
+#define ACL_LVL_DISCLOSE		(ACL_PRIV_DISCLOSE|ACL_LVL_NONE)
+#define ACL_LVL_AUTH			(ACL_PRIV_AUTH|ACL_LVL_DISCLOSE)
 #define ACL_LVL_COMPARE			(ACL_PRIV_COMPARE|ACL_LVL_AUTH)
 #define ACL_LVL_SEARCH			(ACL_PRIV_SEARCH|ACL_LVL_COMPARE)
 #define ACL_LVL_READ			(ACL_PRIV_READ|ACL_LVL_SEARCH)
 #define ACL_LVL_WRITE			(ACL_PRIV_WRITE|ACL_LVL_READ)
+#define ACL_LVL_MANAGE			(ACL_PRIV_MANAGE|ACL_LVL_WRITE)
 
 #define ACL_LVL(m,l)			(((m)&ACL_PRIV_MASK) == ((l)&ACL_PRIV_MASK))
 #define ACL_LVL_IS_NONE(m)		ACL_LVL((m),ACL_LVL_NONE)
+#define ACL_LVL_IS_DISCLOSE(m)	ACL_LVL((m),ACL_LVL_DISCLOSE)
 #define ACL_LVL_IS_AUTH(m)		ACL_LVL((m),ACL_LVL_AUTH)
 #define ACL_LVL_IS_COMPARE(m)	ACL_LVL((m),ACL_LVL_COMPARE)
 #define ACL_LVL_IS_SEARCH(m)	ACL_LVL((m),ACL_LVL_SEARCH)
 #define ACL_LVL_IS_READ(m)		ACL_LVL((m),ACL_LVL_READ)
 #define ACL_LVL_IS_WRITE(m)		ACL_LVL((m),ACL_LVL_WRITE)
+#define ACL_LVL_IS_MANAGE(m)	ACL_LVL((m),ACL_LVL_MANAGE)
 
 #define ACL_LVL_ASSIGN_NONE(m)		ACL_PRIV_ASSIGN((m),ACL_LVL_NONE)
+#define ACL_LVL_ASSIGN_DISCLOSE(m)	ACL_PRIV_ASSIGN((m),ACL_LVL_DISCLOSE)
 #define ACL_LVL_ASSIGN_AUTH(m)		ACL_PRIV_ASSIGN((m),ACL_LVL_AUTH)
 #define ACL_LVL_ASSIGN_COMPARE(m)	ACL_PRIV_ASSIGN((m),ACL_LVL_COMPARE)
 #define ACL_LVL_ASSIGN_SEARCH(m)	ACL_PRIV_ASSIGN((m),ACL_LVL_SEARCH)
 #define ACL_LVL_ASSIGN_READ(m)		ACL_PRIV_ASSIGN((m),ACL_LVL_READ)
 #define ACL_LVL_ASSIGN_WRITE(m)		ACL_PRIV_ASSIGN((m),ACL_LVL_WRITE)
+#define ACL_LVL_ASSIGN_MANAGE(m)	ACL_PRIV_ASSIGN((m),ACL_LVL_MANAGE)
 
 	slap_mask_t	a_access_mask;
 
@@ -1372,6 +1389,7 @@ LDAP_SLAPD_V (int) slapMode;
 #define SLAP_TRUNCATE_MODE	0x0100
 #define	SLAP_TOOL_READMAIN	0x0200
 #define	SLAP_TOOL_READONLY	0x0400
+#define	SLAP_TOOL_QUICK		0x0800
 
 struct slap_replica_info {
 	char *ri_host;				/* supersedes be_replica */
@@ -1447,6 +1465,8 @@ typedef BackendDB Backend;
 #define SLAP_SYNC_RID_SIZE	3
 #define SLAP_SYNCUUID_SET_SIZE 256
 
+#define	SLAP_SYNC_UPDATE_MSGID	2
+
 struct nonpresent_entry {
 	struct berval *npe_name;
 	struct berval *npe_nname;
@@ -1454,9 +1474,8 @@ struct nonpresent_entry {
 };
 
 struct sync_cookie {
-	struct berval *ctxcsn;
-	long sid;
-	struct berval *octet_str;
+	struct berval ctxcsn;
+	struct berval octet_str;
 	long rid;
 	LDAP_STAILQ_ENTRY(sync_cookie) sc_next;
 };
@@ -1466,13 +1485,11 @@ LDAP_STAILQ_HEAD( slap_sync_cookie_s, sync_cookie );
 typedef struct syncinfo_s {
         struct slap_backend_db *si_be;
         long				si_rid;
-        char				*si_provideruri;
-        BerVarray			si_provideruri_bv;
+        struct berval			si_provideruri;
 #define SYNCINFO_TLS_OFF		0
 #define SYNCINFO_TLS_ON			1
 #define SYNCINFO_TLS_CRITICAL	2
         int					si_tls;
-		struct berval		si_updatedn;	
         int					si_bindmethod;
         char				*si_binddn;
         char				*si_passwd;
@@ -1506,11 +1523,14 @@ typedef struct syncinfo_s {
         Avlnode				*si_presentlist;
 		LDAP				*si_ld;
 		LDAP_LIST_HEAD(np, nonpresent_entry) si_nonpresentlist;
-		LDAP_STAILQ_ENTRY( syncinfo_s ) si_next;
 } syncinfo_t;
 
 LDAP_TAILQ_HEAD( be_pcl, slap_csn_entry );
 
+#ifndef SLAP_MAX_CIDS
+#define	SLAP_MAX_CIDS	32	/* Maximum number of supported controls */
+#endif
+
 struct slap_backend_db {
 	BackendInfo	*bd_info;	/* pointer to shared backend info */
 
@@ -1535,6 +1555,7 @@ struct slap_backend_db {
 #define		be_extended	bd_info->bi_extended
 
 #define		be_chk_referrals	bd_info->bi_chk_referrals
+#define		be_chk_controls		bd_info->bi_chk_controls
 #define		be_fetch	bd_info->bi_entry_get_rw
 #define		be_release	bd_info->bi_entry_release_rw
 #define		be_group	bd_info->bi_acl_group
@@ -1548,8 +1569,6 @@ struct slap_backend_db {
  */
 #define		be_has_subordinates bd_info->bi_has_subordinates
 
-#define		be_controls	bd_info->bi_controls
-
 #define		be_connection_init	bd_info->bi_connection_init
 #define		be_connection_destroy	bd_info->bi_connection_destroy
 
@@ -1567,13 +1586,20 @@ struct slap_backend_db {
 #define		be_entry_modify	bd_info->bi_tool_entry_modify
 #endif
 
+	/* supported controls */
+	/* note: set to 0 if the database does not support the control;
+	 * be_ctrls[SLAP_MAX_CIDS] is set to 1 if initialized */
+	char		be_ctrls[SLAP_MAX_CIDS + 1];
+
 /* Database flags */
 #define SLAP_DBFLAG_NOLASTMOD		0x0001U
 #define SLAP_DBFLAG_NO_SCHEMA_CHECK	0x0002U
 #define	SLAP_DBFLAG_GLUE_INSTANCE	0x0010U	/* a glue backend */
 #define	SLAP_DBFLAG_GLUE_SUBORDINATE	0x0020U	/* child of a glue hierarchy */
 #define	SLAP_DBFLAG_GLUE_LINKED		0x0040U	/* child is connected to parent */
-#define SLAP_DBFLAG_OVERLAY			0x0080U	/* this db struct is an overlay */
+#define SLAP_DBFLAG_GLUE_ADVERTISE	0x0080U /* advertise in rootDSE */
+#define SLAP_DBFLAG_OVERLAY		0x0100U	/* this db struct is an overlay */
+#define	SLAP_DBFLAG_GLOBAL_OVERLAY	0x0200U	/* this db struct is a global overlay */
 #define SLAP_DBFLAG_SHADOW		0x8000U /* a shadow */
 #define SLAP_DBFLAG_SYNC_SHADOW		0x1000U /* a sync shadow */
 #define SLAP_DBFLAG_SLURP_SHADOW	0x2000U /* a slurp shadow */
@@ -1590,6 +1616,8 @@ struct slap_backend_db {
 	(SLAP_DBFLAGS(be) & SLAP_DBFLAG_GLUE_SUBORDINATE)
 #define	SLAP_GLUE_LINKED(be)		\
 	(SLAP_DBFLAGS(be) & SLAP_DBFLAG_GLUE_LINKED)
+#define	SLAP_GLUE_ADVERTISE(be)	\
+	(SLAP_DBFLAGS(be) & SLAP_DBFLAG_GLUE_ADVERTISE)
 #define SLAP_SHADOW(be)				(SLAP_DBFLAGS(be) & SLAP_DBFLAG_SHADOW)
 #define SLAP_SYNC_SHADOW(be)			(SLAP_DBFLAGS(be) & SLAP_DBFLAG_SYNC_SHADOW)
 #define SLAP_SLURP_SHADOW(be)			(SLAP_DBFLAGS(be) & SLAP_DBFLAG_SLURP_SHADOW)
@@ -1670,7 +1698,7 @@ struct slap_backend_db {
 	struct		be_pcl	*be_pending_csn_list;
 	ldap_pvt_thread_mutex_t					be_pcl_mutex;
 	ldap_pvt_thread_mutex_t					*be_pcl_mutexp;
-	LDAP_STAILQ_HEAD( be_si, syncinfo_s )	be_syncinfo; /* For syncrepl */
+	syncinfo_t								*be_syncinfo; /* For syncrepl */
 
 	char	*be_realm;
 	void    *be_pb;         /* Netscape plugin */
@@ -1852,6 +1880,8 @@ typedef int (BI_op_extended) LDAP_P((
 	struct slap_op *op, struct slap_rep *rs ));
 typedef int (BI_chk_referrals) LDAP_P((
 	struct slap_op *op, struct slap_rep *rs ));
+typedef int (BI_chk_controls) LDAP_P((
+	struct slap_op *op, struct slap_rep *rs ));
 typedef int (BI_entry_release_rw)
 	LDAP_P(( struct slap_op *op, Entry *e, int rw ));
 typedef int (BI_entry_get_rw) LDAP_P(( struct slap_op *op, struct berval *ndn,
@@ -1953,6 +1983,7 @@ struct slap_backend_info {
 	/* Auxilary Functions */
 	BI_operational		*bi_operational;
 	BI_chk_referrals	*bi_chk_referrals;
+	BI_chk_controls		*bi_chk_controls;
 	BI_entry_get_rw		*bi_entry_get_rw;
 	BI_entry_release_rw	*bi_entry_release_rw;
 
@@ -1996,7 +2027,8 @@ struct slap_backend_info {
 #define SLAP_NOLASTMODCMD(be)	(SLAP_BFLAGS(be) & SLAP_BFLAG_NOLASTMODCMD)
 #define SLAP_LASTMODCMD(be)	(!SLAP_NOLASTMODCMD(be))
 
-	char **bi_controls;		/* supported controls */
+	char	**bi_controls;		/* supported controls */
+	char	bi_ctrls[SLAP_MAX_CIDS + 1];
 
 	unsigned int bi_nDB;	/* number of databases of this type */
 	void	*bi_private;	/* anything the backend type needs */
@@ -2059,35 +2091,6 @@ typedef struct slap_paged_state {
 	int ps_count;
 } PagedResultsState;
 
-#define LDAP_PSEARCH_BY_ADD			0x01
-#define LDAP_PSEARCH_BY_DELETE		0x02
-#define LDAP_PSEARCH_BY_PREMODIFY	0x03
-#define LDAP_PSEARCH_BY_MODIFY		0x04
-#define LDAP_PSEARCH_BY_SCOPEOUT	0x05
-#define LDAP_PSEARCH_BY_PREDELETE	0x06
-
-struct psid_entry {		/* DELETE ME */
-	struct slap_op *ps_op;
-	LDAP_LIST_ENTRY(psid_entry) ps_link;
-};
-
-#if 0	/* DELETE ME */
-struct slog_entry {
-	struct berval sl_uuid;
-	struct berval sl_name;
-	struct berval sl_csn;
-	LDAP_STAILQ_ENTRY(slog_entry) sl_link;
-};
-
-/* session lists */
-struct slap_session_entry {
-	int se_id;
-	int se_size;
-	struct berval se_spec;
-	LDAP_LIST_ENTRY( slap_session_entry ) se_link;
-};
-#endif
-
 struct slap_csn_entry {
 	struct berval ce_csn;
 	unsigned long ce_opid;
@@ -2098,16 +2101,6 @@ struct slap_csn_entry {
 	LDAP_TAILQ_ENTRY (slap_csn_entry) ce_csn_link;
 };
 
-struct pc_entry {
-	ID pc_id;
-	int pc_sent;
-	struct berval pc_csn;
-	struct berval pc_entryUUID;
-	struct berval pc_ename;
-	struct berval pc_enname;
-	LDAP_TAILQ_ENTRY( pc_entry ) pc_link;
-};
-
 /*
  * Caches the result of a backend_group check for ACL evaluation
  */
@@ -2121,10 +2114,6 @@ typedef struct slap_gacl {
 	char ga_ndn[1];
 } GroupAssertion;
 
-#ifndef SLAP_MAX_CIDS
-#define	SLAP_MAX_CIDS	32	/* Maximum number of supported controls */
-#endif
-
 struct slap_control_ids {
 	int sc_assert;
 	int sc_preRead;
@@ -2344,29 +2333,6 @@ typedef struct slap_op {
 
 #define get_pagedresults(op)			((int)(op)->o_pagedresults)
 
-#ifdef BDB_PSEARCH
-	struct sync_cookie	o_sync_state;
-	int					o_sync_rhint;
-	struct berval		o_sync_cid;
-	int					o_sync_slog_size;
-	struct berval		o_sync_csn;
-	struct berval		o_sync_slog_omitcsn;
-	int					o_sync_slog_len;
-	LDAP_STAILQ_HEAD(sl, slog_entry) o_sync_slog_list;
-
-	int o_ps_entries;
-	int	o_no_psearch;
-	LDAP_LIST_ENTRY(slap_op) o_ps_link;
-	LDAP_LIST_HEAD(pe, psid_entry) o_pm_list;
-
-	int o_refresh_in_progress;
-	LDAP_TAILQ_HEAD(pc_pre, pc_entry) o_ps_pre_candidates;
-	LDAP_TAILQ_HEAD(pc_post, pc_entry) o_ps_post_candidates;
-	Avlnode *o_psearch_finished;
-	struct pc_entry *o_ps_send_wait;
-	ldap_pvt_thread_mutex_t	o_pcmutex;
-#endif
-
 	AuthorizationInformation o_authz;
 
 	BerElement	*o_ber;		/* ber of the request */
@@ -2604,8 +2570,8 @@ typedef struct slap_counters_t {
 #define SLAP_CTRL_HIDE				0x80000000U
 #endif
 
-#define SLAP_CTRL_FRONTEND			0x00800000U
-#define SLAP_CTRL_FRONTEND_SEARCH	0x00010000U	/* for NOOP */
+#define SLAP_CTRL_GLOBAL			0x00800000U
+#define SLAP_CTRL_GLOBAL_SEARCH		0x00010000U	/* for NOOP */
 
 #define SLAP_CTRL_OPFLAGS			0x0000FFFFU
 #define SLAP_CTRL_ABANDON			0x00000001U
@@ -2635,11 +2601,6 @@ typedef int (SLAP_CTRL_PARSE_FN) LDAP_P((
 #define SLAP_ZONE_ALLOC 1
 #undef SLAP_ZONE_ALLOC
 
-#if defined(LDAP_DEVEL) && defined(ENABLE_REWRITE)
-/* use librewrite for sasl-regexp */
-#define SLAP_AUTH_REWRITE	1
-#endif /* LDAP_DEVEL && ENABLE_REWRITE */
-
 #ifdef LDAP_COMP_MATCH
 /*
  * Extensible Filter Definition
