@@ -1,8 +1,8 @@
 /* slapcommon.h - common definitions for the slap tools */
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapcommon.h,v 1.7 2004/12/06 14:17:23 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapcommon.h,v 1.7.2.1 2005/01/20 18:03:56 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,21 +32,10 @@ enum slaptool {
 	SLAPLAST
 };
 
-#define SLAP_TOOL_CTXCSN_KEEP	0
-#define SLAP_TOOL_CTXCSN_ENTRY	1
-#define SLAP_TOOL_CTXCSN_BATCH	2
-
 typedef struct tool_vars {
 	Backend *tv_be;
 	int tv_verbose;
 	int tv_update_ctxcsn;
-	int tv_retrieve_ctxcsn;
-	int tv_retrieve_synccookie;
-	int tv_replica_promotion;
-	int tv_replica_demotion;
-	char    *tv_replica_id_string;
-	char    **tv_replica_id_strlist;
-	int     *tv_replica_id_list;
 	int tv_continuemode;
 	int tv_nosubordinates;
 	int tv_dryrun;
@@ -66,13 +55,6 @@ extern tool_vars tool_globals;
 #define	be tool_globals.tv_be
 #define verbose tool_globals.tv_verbose
 #define update_ctxcsn tool_globals.tv_update_ctxcsn
-#define retrieve_ctxcsn tool_globals.tv_retrieve_ctxcsn
-#define retrieve_synccookie tool_globals.tv_retrieve_synccookie
-#define replica_promotion tool_globals.tv_replica_promotion
-#define replica_demotion tool_globals.tv_replica_demotion
-#define replica_id_string tool_globals.tv_replica_id_string
-#define replica_id_strlist tool_globals.tv_replica_id_strlist
-#define replica_id_list tool_globals.tv_replica_id_list
 #define continuemode tool_globals.tv_continuemode
 #define nosubordinates tool_globals.tv_nosubordinates
 #define dryrun tool_globals.tv_dryrun
